# Functions for Analysis


# Evaluates the "score" of the current community extracted from graph grph_tmp
evaluate <- function(th) {
  # evaluate the tabu for a value of th
  # tabuSearch will use this function to evaluate points in its search space
  # if everything is turned off just return zero as we are not interested
  if(sum(th) == 0 | sum(th) == n)  return(0)
  
  # we want to identify the subgraph thats the current binary string
  # sub_ind <- colnames(th)[which(th == 1)]
  ind <- which(th == 1)
  sub_ind <- V_indx[ind]
  sub_comm = induced_subgraph(grph_tmp, as.character(sub_ind)) #indexes of vertices in extracted community
  non_comm <- V_indx[!V_indx %in% sub_ind] #indexes of vertices in compliment to extracted community
  # print(non_comm)
  val_S <- length(V(sub_comm)) #number of vertices in extracted community
  val_S_comp <- length(non_comm) #number of vertices in complement to extracted community
  
  # This gives all edges that originate from nodes in 
  #community
  sub_comm_edges <- E(grph_tmp)[inc(V(grph_tmp)[as.character(sub_ind)])]
  
  # Convert to a matrix
  sub_comm_edges_m <- get.edges(grph_tmp, sub_comm_edges) #matrix representation
  
  # Then filter out edges that are completely only in extracted community
  sub_comm_only <- sub_comm_edges[!(
    sub_comm_edges_m[, 1] %in% V(grph_tmp)[as.character(sub_ind)] & 
      sub_comm_edges_m[, 2] %in% V(grph_tmp)[as.character(sub_ind)]
  )] # filter where in col1 and col2
  
  # gets all weights associated with edges that connect extracted community
  # to rest of network
  sub_comm_weights <- edge_attr(grph_tmp, "weight", sub_comm_only) 
  
  
  SXSCompl <- val_S*val_S_comp #This is |S|*|Sc|
  
  # Normalizing extracted community by its size gives
  norm.comm <- (2*sum(E(sub_comm)$weight))/(val_S^2)
  
  norm.Scomp <- (2*sum(sub_comm_weights))/(SXSCompl)
  
  # Objective function value 
  obj_fun <- SXSCompl*(norm.comm-norm.Scomp)
  # print(obj_fun)
  return(obj_fun)
}

# Returns the doc index and community lables if given
# list is doc indexes split along their cuts
# Entire correlation matrix
# The cut_number

indxTOcommFUN <- function(list_docIND, corr, cut_no){
  new_agg_list <- list()
  cnt = 0
  run_b <- length(list_docIND)
  for (b in 1: run_b){
    # b = 3
    cnt = 0
    new_comm <- list()
    n = length(list_docIND[[b]])
    for (i in 1:n){
      # i = 1
      cnt = cnt + 1
      a = list_docIND[[b]][[i]]
      vec1 <- (b-1)*cut_no + a
      new_comm[[cnt]] <- vec1
      dens_1 <- 0.85 * mean(corr[vec1, vec1])
      for (j in 1:n) {
        c = list_docIND[[b]][[j]]
        if(i != j){
          vec2 <- (b-1)*cut_no + c
          dens_2 <- 0.85 * mean(corr[vec2, vec2])
          check_comm <- mean(corr[vec1, vec2])
          if( (dens_1 < check_comm) | (dens_2 < check_comm)) {
            new_comm[[cnt]] <- sort(unique(c(vec1,vec2)))
          }
        }
      }
    }
    new_agg_list[[b]] <- new_comm
  }
  
  for ( ind1 in 1:run_b){
    n = length(new_agg_list[[ind1]])
    for ( i in 1:n) {
      vec1 <- new_agg_list[[ind1]][[i]]
      dens_1 <- 0.85 * mean(corr[vec1, vec1])
      for (b in 1: run_b){
        p <- length(new_agg_list[[b]])
        for (j in 1:p){
          vec2 <- new_agg_list[[b]][[j]]
          dens_2 <- 0.85 * mean(corr[vec2, vec2])
          check_comm <- mean(corr[vec1, vec2])
          if(dens_2 > 0.2) {
            if( (dens_1 < check_comm) | (dens_2 < check_comm)) {
              new_agg_list[[ind1]][[i]] <- sort(unique(c(new_agg_list[[ind1]][[i]],vec2)))
            }
          }
        }
      }
    }
  }
  
  
  mat_tmp <- as.data.frame(matrix(0, nrow = 1, ncol = 100))
  for (i in 1:length(new_agg_list)){
    tmp <- as.data.frame(t((do.call(cbind.fill, new_agg_list[[i]]))))
    tmp <- tmp[!duplicated(tmp),]
    mat_tmp <- rbind.fill(mat_tmp, tmp)
  }
  
  mat_tmp <- mat_tmp[-1,]
  
  # First determine number of total documents assigned to a community
  doc_lnths <- apply(mat_tmp, 1, function(x) sum(!is.na(x)))
  tot_docs <- sum(doc_lnths)
  
  res_mat <- matrix(0, nrow = tot_docs, ncol = 2)
  strt <- 1
  endn <- doc_lnths[1]
  rnge <- c(strt:endn)
  for (i in 1: nrow(mat_tmp)){
    doc_IDs <- as.numeric(mat_tmp[i,!is.na(mat_tmp[i,])])
    res_mat[rnge,1] <- doc_IDs
    res_mat[rnge,2] <- i
    if(i < nrow(mat_tmp)){
      strt <- sum(doc_lnths[1:i]) + 1
      endn <- sum(doc_lnths[1:(i+1)])
      rnge <- c(strt:endn)
    }
  }
  
  x1 <- which(duplicated(res_mat[,1]) == T)
  for (i in 1:length(x1)){ #x1
    ind1 <- which(res_mat[,1] == res_mat[x1[i],1])
    comm1 <- res_mat[ind1[1],2]
    for (j in 2:length(ind1)){
      # j = 3
      res_mat[which(res_mat[,2] == res_mat[ind1[j], 2]),2] <- comm1
    }
  }
  
  res <- res_mat[!duplicated(res_mat),]
  return(res)
  
}

indxTOcommFUN2 <- function(list_docIND, corr, cut_no){
  # list_docIND <- doc_Vaccine[[1]]
  # corr <- corr_LSA_Vaccine
  # corr[corr < 0] <- 0
  new_agg_list <- list()
  cnt = 0
  run_b <- length(list_docIND)
  for (b in 1: run_b){
    # b = 1
    cnt = 0
    new_comm <- list()
    n = length(list_docIND[[b]])
    for (i in 1:n){
      # i = 1
      cnt = cnt + 1
      a = list_docIND[[b]][[i]]
      vec1 <- a
      new_comm[[cnt]] <- vec1
      dens_1 <- 0.8 * mean(corr[vec1, vec1])
      # print(paste("This is dense 1 ", round(dens_1, digits = 2)))
      for (j in 1:n) {
        # j = 1
        c = list_docIND[[b]][[j]]
        if(i != j){
          vec2 <-  c
          dens_2 <- 0.8 * mean(corr[vec2, vec2])
          check_comm <- mean(corr[vec1, vec2])
          # print(paste("This is dense 2 ", round(dens_2, digits = 2)))
          if( (dens_1 < check_comm) & (dens_2 < check_comm)) {
            # print(paste("Combine communities dense 1", round(dens_1, digits = 2), "Dense 2 ",round(dens_2, digits = 2) ))
            new_comm[[cnt]] <- sort(unique(c(vec1,vec2)))
          }
        }
      }
    }
    new_agg_list[[b]] <- new_comm
  }
  for ( ind1 in 1:run_b){
    # ind1 <- 1
    n = length(new_agg_list[[ind1]])
    for ( i in 1:n) {
      # i = 1
      vec1 <- new_agg_list[[ind1]][[i]]
      dens_1 <- 0.8 * mean(corr[vec1, vec1])
      for (b in 1: run_b){
        # b = 1
        p <- length(new_agg_list[[b]])
        for (j in 1:p){
          # j = 2
          vec2 <- new_agg_list[[b]][[j]]
          dens_2 <- 0.8 * mean(corr[vec2, vec2])
          check_comm <- mean(corr[vec1, vec2])
          # print(paste("This is i with other comms", round(check_comm, digits = 2)))
          if(dens_2 > 0.2) {
            if( (dens_1 < check_comm) & (dens_2 < check_comm)) {
              print(paste("Combine communities dense 1", round(dens_1, digits = 2), "Dense 2 ",round(dens_2, digits = 2) ))
              print(paste("Index i", i, "and Run 1 is", ind1, "Index j", j, "and Run 2 is", b))
              new_agg_list[[ind1]][[i]] <- sort(unique(c(new_agg_list[[ind1]][[i]],vec2)))
            }
          }
        }
      }
    }
  }

  mat_tmp <- as.data.frame(matrix(0, nrow = 1, ncol = 100))
  for (i in 1:length(new_agg_list)){
    tmp <- as.data.frame(t((do.call(cbind.fill, new_agg_list[[i]]))))
    tmp <- tmp[!duplicated(tmp),]
    mat_tmp <- rbind.fill(mat_tmp, tmp)
  }
  
  mat_tmp <- mat_tmp[-1,]
  
  # First determine number of total documents assigned to a community
  doc_lnths <- apply(mat_tmp, 1, function(x) sum(!is.na(x)))
  tot_docs <- sum(doc_lnths)
  
  res_mat <- matrix(0, nrow = tot_docs, ncol = 2)
  strt <- 1
  endn <- doc_lnths[1]
  rnge <- c(strt:endn)
  for (i in 1: nrow(mat_tmp)){
    doc_IDs <- as.numeric(mat_tmp[i,!is.na(mat_tmp[i,])])
    res_mat[rnge,1] <- doc_IDs
    res_mat[rnge,2] <- i
    if(i < nrow(mat_tmp)){
      strt <- sum(doc_lnths[1:i]) + 1
      endn <- sum(doc_lnths[1:(i+1)])
      rnge <- c(strt:endn)
    }
  }
  
  x1 <- which(duplicated(res_mat[,1]) == T)
  for (i in 1:length(x1)){ #x1
    ind1 <- which(res_mat[,1] == res_mat[x1[i],1])
    comm1 <- res_mat[ind1[1],2]
    for (j in 2:length(ind1)){
      # j = 3
      res_mat[which(res_mat[,2] == res_mat[ind1[j], 2]),2] <- comm1
    }
  }
  
  res <- res_mat[!duplicated(res_mat),]
  return(res)
  
}

mat_docs <- function(docs, samps, indx_mat){
  mat <- matrix("", nrow = 2, ncol = 3)
  mat[1,2] <- docs[indx_mat[samps,1]]
  mat[2,2] <- docs[indx_mat[samps,2]]
  mat[1,1] <- as.character(indx_mat[samps,1])
  mat[2,1] <- as.character(indx_mat[samps,2])
  mat[1,3] <- "1"
  mat[2,3] <- "2"
  return(mat)
}

rmve_words <- function(text, rmve) {
  words <- strsplit(text, " ")[[1]]
  txt <- paste(words[!words %in% rmve], collapse = " ")
  return(txt)
}
comm_words = function(mat, customStopWords){
  txt = VectorSource(mat)
  corpus2 = VCorpus(txt)
  corpus2 = tm_map(corpus2,removePunctuation)
  corpus2 = tm_map(corpus2, removeWords, customStopWords) #remove stopwords
  corpus2 = tm_map(corpus2, removeWords, stopwords("en")) #remove stopwords
  corpus2 = tm_map(corpus2,stemDocument)
  corpus3<-tm_map(corpus2, content_transformer( function(x) clean_stemmedText(x)))
  
  myDf_Corpus <- data.frame(text = sapply(corpus2, paste, collapse = " "), stringsAsFactors = FALSE)
  myDf_Corpus = myDf_Corpus[which(myDf_Corpus!= ""),]
  
  my_words = str_split(myDf_Corpus, pattern = " ")
  val_fn = as.matrix(table(unlist(my_words)))
  val_fn = (cbind(val_fn, row.names(val_fn)))
  df_valFN = as.data.frame(val_fn)
  df_valFN[,1] = as.numeric(as.character(df_valFN[,1]))
  tmp <- df_valFN[order(df_valFN$V1, decreasing = T),]
  
  # max( df_valFN[,1])
  if(length(which(tmp[,2] == "")) > 0){
    tmp = tmp[-which(tmp[,2] == ""),]
  }
  col2_nm = paste(tmp[1:4,2], collapse = ", ")
  return(col2_nm)
}

# Functions to load 
cbind.fill <- function(...){
  nm <- list(...) 
  nm <- lapply(nm, as.matrix)
  n <- max(sapply(nm, nrow)) 
  do.call(cbind, lapply(nm, function (x) 
    rbind(x, matrix(, n-nrow(x), ncol(x))))) 
}


name_colm = function(mat, customStopWords){
  #mat = res_bindMat
  txt = VectorSource(mat[,2])
  corpus2 = VCorpus(txt)
  corpus2 = tm_map(corpus2,removePunctuation)
  corpus2 = tm_map(corpus2, removeWords, stopwords('english')) #remove stopwords
  #corpus2 = tm_map(corpus2, removeWords, customStopWords)
  corpus3<-tm_map(corpus2, content_transformer( function(x) clean_stemmedText(x)))
  
  myDf_Corpus <- data.frame(text = sapply(corpus3, paste, collapse = " "), stringsAsFactors = FALSE)
  myDf_Corpus = myDf_Corpus[which(myDf_Corpus!= ""),]
  
  my_words = str_split(myDf_Corpus, pattern = " ")
  val_fn = as.matrix(table(unlist(my_words)))
  val_fn = cbind(val_fn, row.names(val_fn))
  val_fn = val_fn[order(val_fn[,1], decreasing = T),]
  val_fn = val_fn[-which(val_fn[,2] == ""),]
  col2_nm = paste(val_fn[1:3,2], collapse = ", ")
  colnames(mat) = c("Doc ID", col2_nm)
  return(mat)
}
clean_stemmedText <- function(txt){
  txt = str_replace_all(txt, "\\bbabd\\b", "abdomen ")
  txt = str_replace_all(txt, "\\babbreviationexpect\\b", " abbreviation expect ")
  txt = str_replace_all(txt, "\\babdomenom\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babdomenomin\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babdomenomen\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babdomin\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babdomenom\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babdomenomenomen\\b", " abdomen ")
  txt = str_replace_all(txt, "\\baccid\\b", " accident ")
  txt = str_replace_all(txt, "\\baccid\\b", " accident ")
  txt = str_replace_all(txt, "\\baccus\\b", " accuse ")
  txt = str_replace_all(txt, "\\baccusatori\\b", " accuse ")
  txt = str_replace_all(txt, "\\backnowedg\\b", " acknowledge ")
  txt = str_replace_all(txt, "\\backnowledg\\b", " acknowledge ")
  txt = str_replace_all(txt, "\\bacrossfrom\\b", " across from ")
  txt = str_replace_all(txt, "\\badequ\\b", " adequate ")
  txt = str_replace_all(txt, "\\badjac\\b", " adjacent ")
  txt = str_replace_all(txt, "\\badminist\\b", " administer ")
  txt = str_replace_all(txt, "\\badministr\\b", " administer ")
  txt = str_replace_all(txt, "\\badminst\\b", " administer ")
  txt = str_replace_all(txt, "\\badminstr\\b", " administer ")
  txt = str_replace_all(txt, "\\badminstr\\b", " administer ")
  txt = str_replace_all(txt, "\\baftger\\b", " after ")
  txt = str_replace_all(txt, "\\ballergi\\b", " allergy ")
  txt = str_replace_all(txt, "\\baftger\\b", " administer ")
  txt = str_replace_all(txt, "\\baftger\\b", " administer ")
  txt = str_replace_all(txt, "\\bordr\\b", " order ")
  txt = str_replace_all(txt, "\\borderd\\b", " order ")
  txt = str_replace_all(txt, "\\bordc\\b", " order dc ")
  txt = str_replace_all(txt, "\\borderdc\\b", " order dc ")
  txt = str_replace_all(txt, "\\bordered\\b", " order ")
  txt = str_replace_all(txt, "\\borderedtim\\b", " order time ")
  txt = str_replace_all(txt, "\\borderingdr\\b", " order dr ")
  txt = str_replace_all(txt, "\\borderpt\\b", " order pt ")
  txt = str_replace_all(txt, "\\bordersday\\b", " order day ")
  txt = str_replace_all(txt, "\\bordersth\\b", " order the ")
  txt = str_replace_all(txt, "\\borderth\\b", " order the ")
  txt = str_replace_all(txt, "\\breordererd\\b", " reorder ")
  txt = str_replace_all(txt, "\\bappropri\\b", " appropriate ")
  txt = str_replace_all(txt, "\\bemerdr\\b", " emergency dr ")
  txt = str_replace_all(txt, "\\bacess\\b", " access ")
  
  txt = str_replace_all(txt, "\\baddend\\b", " addendum ")
  txt = str_replace_all(txt, "\\babscesus\\b", " abscess ")
  txt = str_replace_all(txt, "\\badministerd\\b", " administer ")
  txt = str_replace_all(txt, "\\badministrt\\b", " administer ")
  txt = str_replace_all(txt, "\\bascess\\b", " access ")
  
  txt = str_replace_all(txt, "\\bcontast\\b", " contact ")
  txt = str_replace_all(txt, "\\bcosignatur\\b", " cosign ")
  txt = str_replace_all(txt, "\\bmedicaiton\\b", " medication ")
  txt = str_replace_all(txt, "\\bordr\\b", " order ")
  txt = str_replace_all(txt, "\\bpharmci\\b", " pharmacy ")
  txt = str_replace_all(txt, "\\bquantit\\b", " quantity ")
  txt = str_replace_all(txt, "\\bquantiti\\b", " quantity ")
  txt = str_replace_all(txt, "\\breadmiss\\b", " read miss ")
  txt = str_replace_all(txt, "\\bshouldn\\b", " should not ")
  txt = str_replace_all(txt, "\\bsometh\\b", " somethin ")
  txt = str_replace_all(txt, "\\bsresid\\b", " reside ")
  txt = str_replace_all(txt, "\\bthur\\b", " thursday ")
  txt = str_replace_all(txt, "\\btranfer\\b", " transfer ")
  txt = str_replace_all(txt, "\\btraumat\\b", " trauma ")
  txt = str_replace_all(txt, "\\bfri\\b", " friday ")
  txt = str_replace_all(txt, "\\bhadn\\b", " had not ")
  txt = str_replace_all(txt, "\\bhrorder\\b", " hr order ")
  txt = str_replace_all(txt, "\\bitchi\\b", " itch ")
  txt = str_replace_all(txt, "\\bokay\\b", " ok ")
  txt = str_replace_all(txt, "\\bproced\\b", " proceed ")
  txt = str_replace_all(txt, "\\bthru\\b", " through ")
  txt = str_replace_all(txt, "\\brepres\\b", " represent ")
  txt = str_replace_all(txt, "\\bdoseag\\b", " dose ")
  txt = str_replace_all(txt, "\\binpt\\b", " in pt ")
  txt = str_replace_all(txt, "\\bnecessaryth\\b", " necessari ")
  txt = str_replace_all(txt, "\\bnecessarili\\b", " necessari ")
  txt = str_replace_all(txt, "\\bnecess\\b", " necessari ")
  txt = str_replace_all(txt, "\\borderd\\b", " order ")
  txt = str_replace_all(txt, "\\bstabal\\b", " stabl ")
  txt = str_replace_all(txt, "\\bstabil\\b", " stabl ")
  
  txt = str_replace_all(txt, "\\boverridden\\b", " overrid ")
  txt = str_replace_all(txt, "\\boverrod\\b", " overrid ")
  txt = str_replace_all(txt, "\\boverriden\\b", " overrid ")
  txt = str_replace_all(txt, "\\bchosen\\b", " choose ")
  txt = str_replace_all(txt, "\\bchoos\\b", " choose ")
  txt = str_replace_all(txt, "\\bchose\\b", " choose ")
  
  txt = str_replace_all(txt, "\\burinalysi\\b", " urin analysis ")
  txt = str_replace_all(txt, "\\bformul\\b", " formula ")
  txt = str_replace_all(txt, "\\bshown\\b", " show ")
  txt = str_replace_all(txt, "\\bnovolog\\b", " novalog ")
  txt = str_replace_all(txt, "\\badmiss\\b", " admission ")
  txt = str_replace_all(txt, "\\badmitt\\b", " admit ")
  txt = str_replace_all(txt, "\\badvi\\b", " advise ")
  txt = str_replace_all(txt, "\\badvic\\b", " advise ")
  txt = str_replace_all(txt, "\\badvoc\\b", " advocate ")
  txt = str_replace_all(txt, "\\badress\\b", " address ")
  txt = str_replace_all(txt, "\\badress\\b", " address ")
  
  txt = str_replace_all(txt, "\\badver\\b", " adverse ")
  txt = str_replace_all(txt, "\\bagit\\b", " agitate ")
  txt = str_replace_all(txt, "\\balarn\\b", " alarm ")
  txt = str_replace_all(txt, "\\ballerg\\b", " allergy ")
  txt = str_replace_all(txt, "\\ballergi\\b", " allergy ")
  txt = str_replace_all(txt, "\\bambbbul\\b", " ambulate ")
  txt = str_replace_all(txt, "\\bambu\\b", " ambulate ")
  txt = str_replace_all(txt, "\\banesthesai\\b", " anesthesia ")
  txt = str_replace_all(txt, "\\banestheshia\\b", " anesthesia ")
  txt = str_replace_all(txt, "\\banethesia\\b", " anesthesia ")
  txt = str_replace_all(txt, "\\banesthesiologist\\b", " anesthesiologist ")
  txt = str_replace_all(txt, "\\banesthiologist\\b", " anesthesiologist ")
  txt = str_replace_all(txt, "\\banesthologist\\b", " anesthesiologist ")
  txt = str_replace_all(txt, "\\banethesiologist\\b", " anesthesiologist ")
  txt = str_replace_all(txt, "\\banthesiologist\\b", " anesthesiologist ")
  txt = str_replace_all(txt, "\\banywh\\b", " any where ")
  txt = str_replace_all(txt, "\\bappliedth\\b", " applied the ")
  txt = str_replace_all(txt, "\\bappoxim\\b", " approximate ")
  txt = str_replace_all(txt, "\\bapprop\\b", " appropriate ")
  txt = str_replace_all(txt, "\\bappropr\\b", " appropriate ")
  txt = str_replace_all(txt, "\\bapprox\\b", " approximate ")
  txt = str_replace_all(txt, "\\bapproxim\\b", " approximate ")
  txt = str_replace_all(txt, "\\bapprx\\b", " approximate ")
  txt = str_replace_all(txt, "\\bappx\\b", " approximate ")
  txt = str_replace_all(txt, "\\baprox\\b", " approximate ")
  txt = str_replace_all(txt, "\\bargument\\b", " argue ")
  txt = str_replace_all(txt, "\\bargu\\b", " argue ")
  txt = str_replace_all(txt, "\\bassit\\b", " assist ")
  txt = str_replace_all(txt, "\\basst\\b", " assist ")
  txt = str_replace_all(txt, "\\battemt\\b", " attempt ")
  
  txt = str_replace_all(txt, "\\battent\\b", " attention ")
  txt = str_replace_all(txt, "\\bbaseln\\b", " baselin ")
  txt = str_replace_all(txt, "\\bbathrroom\\b", " bathroom ")
  txt = str_replace_all(txt, "\\bbathroompt\\b", " bathroom ")
  txt = str_replace_all(txt, "\\bbathroomwith\\b", " bathroom with ")
  txt = str_replace_all(txt, "\\bbcame\\b", " become ")
  txt = str_replace_all(txt, "\\bbecam\\b", " become ")
  txt = str_replace_all(txt, "\\bbecom\\b", " become ")
  txt = str_replace_all(txt, "\\bbenadri\\b", " benadryl ")
  txt = str_replace_all(txt, "\\bbps\\b", " bp ")
  
  
  txt = str_replace_all(txt, "\\bbradycard\\b", " bradycardiac ")
  txt = str_replace_all(txt, "\\bbradycardia\\b", " bradycardiac ")
  txt = str_replace_all(txt, "\\bbreath\\b", " breadth ")
  txt = str_replace_all(txt, "\\bbriefli\\b", " brief ")
  txt = str_replace_all(txt, "\\bcalcul\\b", " calculate ")
  txt = str_replace_all(txt, "\\bcalll\\b", " call ")
  txt = str_replace_all(txt, "\\bcapac\\b", " capacity ")
  txt = str_replace_all(txt, "\\bcathet\\b", " catheter ")
  txt = str_replace_all(txt, "\\bchang\\b", " change ")
  
  txt = str_replace_all(txt, "\\bchemo\\b", " chemotherapi ")
  txt = str_replace_all(txt, "\\bchildren\\b", " child ")
  txt = str_replace_all(txt, "\\bcircul\\b", " circulate ")
  txt = str_replace_all(txt, "\\bcollabor\\b", " collaborate ")
  txt = str_replace_all(txt, "\\bcommod\\b", " commode ")
  txt = str_replace_all(txt, "\\bcommunic\\b", " communicate ")
  txt = str_replace_all(txt, "\\bcomplaint\\b", " complain ")
  txt = str_replace_all(txt, "\\bcompromi\\b", " compromise ")
  txt = str_replace_all(txt, "\\bconcentr\\b", " concentrate ")
  txt = str_replace_all(txt, "\\bcondit\\b", " condition ")
  txt = str_replace_all(txt, "\\bconditionh\\b", " condition ")
  txt = str_replace_all(txt, "\\bcontin\\b", " continue ")
  txt = str_replace_all(txt, "\\bcontinu\\b", " continue ")
  txt = str_replace_all(txt, "\\bcontinuin\\b", " continue ")
  txt = str_replace_all(txt, "\\bcontinuo\\b", " continue ")
  txt = str_replace_all(txt, "\\bcontinuosli\\b", " continue ")
  txt = str_replace_all(txt, "\\bconver\\b", " convert ")
  txt = str_replace_all(txt, "\\bcoordin\\b", " coordinate ")
  txt = str_replace_all(txt, "\\bdaugther\\b", " daughter ")
  txt = str_replace_all(txt, "\\bdecrea\\b", " decrease ")
  
  txt = str_replace_all(txt, "\\bdeepli\\b", " deeply ")
  txt = str_replace_all(txt, "\\bdeliv\\b", " deliver ")
  txt = str_replace_all(txt, "\\bdeliveri\\b", " deliver ")
  txt = str_replace_all(txt, "\\bdiagno\\b", " diagnose ")
  txt = str_replace_all(txt, "\\bdiagnost\\b", " diagnose ")
  txt = str_replace_all(txt, "\\bdiagnosi\\b", " diagnose ")
  txt = str_replace_all(txt, "\\bdidn\\b", " did not ")
  txt = str_replace_all(txt, "\\bdidnt\\b", " did not ")
  txt = str_replace_all(txt, "\\bdiff\\b", " different ")
  txt = str_replace_all(txt, "\\bdiffer\\b", " different ")
  txt = str_replace_all(txt, "\\bdifferenti\\b", " different ")
  txt = str_replace_all(txt, "\\bdiffernet\\b", " different ")
  txt = str_replace_all(txt, "\\bdifficult\\b", " difficult ")
  txt = str_replace_all(txt, "\\bdifficulti\\b", " difficult ")
  txt = str_replace_all(txt, "\\bdigoxen\\b", " digoxin ")
  txt = str_replace_all(txt, "\\bdiscrep\\b", " discrepancy ")
  txt = str_replace_all(txt, "\\bdiscrepancynew\\b", " discrepancy new ")
  txt = str_replace_all(txt, "\\bdiscrepencywa\\b", " discrepancy was ")
  txt = str_replace_all(txt, "\\bdiscrimin\\b", " discriminate ")
  txt = str_replace_all(txt, "\\bdisloc\\b", " dislocate ")
  txt = str_replace_all(txt, "\\bdisreagard\\b", " disregard ")
  txt = str_replace_all(txt, "\\bdoceument\\b", " deeply ")
  txt = str_replace_all(txt, "\\bdocuent\\b", " deeply ")
  txt = str_replace_all(txt, "\\bdocumentaion\\b", " deeply ")
  txt = str_replace_all(txt, "\\bdoen\\b", " does not ")
  txt = str_replace_all(txt, "\\bdoesnt\\b", " does not ")
  txt = str_replace_all(txt, "\\bdoesn\\b", " does not ")
  txt = str_replace_all(txt, "\\bdownunderwear\\b", " down underwear ")
  txt = str_replace_all(txt, "\\bdrew\\b", " draw ")
  txt = str_replace_all(txt, "\\bdrawn\\b", " draw ")
  txt = str_replace_all(txt, "\\bemerg\\b", " emergency ")
  txt = str_replace_all(txt, "\\bemergeci\\b", " emergency ")
  txt = str_replace_all(txt, "\\bemphat\\b", " emphatic ")
  txt = str_replace_all(txt, "\\bemrgenc\\b", " emergency ")
  txt = str_replace_all(txt, "\\berextend\\b", " er extend ")
  txt = str_replace_all(txt, "\\berron\\b", " error ")
  txt = str_replace_all(txt, "\\berrorbut\\b", " error but ")
  txt = str_replace_all(txt, "\\berrorphysician\\b", " error physician ")
  txt = str_replace_all(txt, "\\bescal\\b", " escalate ")
  txt = str_replace_all(txt, "\\besrdreciev\\b", " esrd reciev ")
  txt = str_replace_all(txt, "\\bessenti\\b", " essential ")
  txt = str_replace_all(txt, "\\bevalu\\b", " evaluate ")
  txt = str_replace_all(txt, "\\beval\\b", " evaluate ")
  txt = str_replace_all(txt, "\\bevaul\\b", " evaluate ")
  txt = str_replace_all(txt, "\\bexmain\\b", " examin ")
  txt = str_replace_all(txt, "\\bexamein\\b", " examin ")
  txt = str_replace_all(txt, "\\bexperi\\b", " experienc ")
  txt = str_replace_all(txt, "\\bexplaind\\b", " explain ")
  txt = str_replace_all(txt, "\\bexplan\\b", " explain ")
  txt = str_replace_all(txt, "\\bfacil\\b", " facility ")
  txt = str_replace_all(txt, "\\bfacili\\b", " facility ")
  txt = str_replace_all(txt, "\\bfaciliit\\b", " facility ")
  txt = str_replace_all(txt, "\\bfacilit\\b", " facility ")
  txt = str_replace_all(txt, "\\bfallen\\b", " fall ")
  txt = str_replace_all(txt, "\\bfell\\b", " fall ")
  txt = str_replace_all(txt, "\\bfloorreport\\b", " floor report ")
  txt = str_replace_all(txt, "\\bflloor\\b", " floor ")
  txt = str_replace_all(txt, "\\bfloorv\\b", " floor ")
  txt = str_replace_all(txt, "\\bfollowup\\b", " follow up ")
  txt = str_replace_all(txt, "\\bforgot\\b", " forget ")
  txt = str_replace_all(txt, "\\bforgotten\\b", " forget ")
  txt = str_replace_all(txt, "\\bheadahc\\b", " headach ")
  txt = str_replace_all(txt, "\\bheparim\\b", " heparin ")
  txt = str_replace_all(txt, "\\bhepain\\b", " heparin ")
  txt = str_replace_all(txt, "\\bheparindc\\b", " heparin dc ")
  txt = str_replace_all(txt, "\\bhous\\b", " house ")
  txt = str_replace_all(txt, "\\bhousekeep\\b", " house keep ")
  txt = str_replace_all(txt, "\\bhouskeep\\b", " house keep ")
  txt = str_replace_all(txt, "\\bhrs\\b", " hr ")
  txt = str_replace_all(txt, "\\bidenrtif\\b", " identify ")
  txt = str_replace_all(txt, "\\bidentif\\b", " identify ")
  txt = str_replace_all(txt, "\\bidentifi\\b", " identify ")
  
  txt = str_replace_all(txt, "\\bimmedi\\b", " immediate ")
  txt = str_replace_all(txt, "\\bimmediat\\b", " immediate ")
  txt = str_replace_all(txt, "\\bimmediateley\\b", " immediate ")
  txt = str_replace_all(txt, "\\bimmediatley\\b", " immediate ")
  txt = str_replace_all(txt, "\\binfiltrart\\b", " infiltrate ")
  txt = str_replace_all(txt, "\\binfiltr\\b", " infiltrate ")
  txt = str_replace_all(txt, "\\binfomr\\b", " inform ")
  txt = str_replace_all(txt, "\\biniti\\b", " initial ")
  txt = str_replace_all(txt, "\\binit\\b", " initiate ")
  txt = str_replace_all(txt, "\\binjectionand\\b", " inject and ")
  txt = str_replace_all(txt, "\\binjur\\b", " injury ")
  txt = str_replace_all(txt, "\\binjuri\\b", " injury ")
  txt = str_replace_all(txt, "\\binpati\\b", " in patient ")
  txt = str_replace_all(txt, "\\binsertionpt\\b", " insert pt ")
  txt = str_replace_all(txt, "\\bintermitt\\b", " intermitent ")
  txt = str_replace_all(txt, "\\bintermitten\\b", " intermitent ")
  txt = str_replace_all(txt, "\\binterviw\\b", " interview ")
  txt = str_replace_all(txt, "\\bknown\\b", " know ")
  txt = str_replace_all(txt, "\\blabeld\\b", " label ")
  txt = str_replace_all(txt, "\\blaboratori\\b", " laboratory ")
  txt = str_replace_all(txt, "\\blabratori\\b", " laboratory ")
  txt = str_replace_all(txt, "\\bmononitr\\b", " monitor ")
  txt = str_replace_all(txt, "\\bmonitr\\b", " monitor ")
  txt = str_replace_all(txt, "\\bmorn\\b", " morning ")
  txt = str_replace_all(txt, "\\bnegat\\b", " negative ")
  txt = str_replace_all(txt, "\\bneg\\b", " negative ")
  txt = str_replace_all(txt, "\\bnotic\\b", " notify ")
  txt = str_replace_all(txt, "\\bnotif\\b", " notify ")
  txt = str_replace_all(txt, "\\bnotifeid\\b", " notify ")
  txt = str_replace_all(txt, "\\bnotifi\\b", " notify ")
  txt = str_replace_all(txt, "\\bnotifiedpati\\b", " notify patient ")
  txt = str_replace_all(txt, "\\bnotifiedphysician\\b", " notify physician ")
  txt = str_replace_all(txt, "\\bnotifiedthi\\b", " identify this ")
  txt = str_replace_all(txt, "\\bnotiif\\b", " notify ")
  txt = str_replace_all(txt, "\\bnumber\\b", " num ")
  txt = str_replace_all(txt, "\\bnumer\\b", " num ")
  txt = str_replace_all(txt, "\\bnur\\b", " nurse ")
  txt = str_replace_all(txt, "\\bnurisng\\b", " nurse ")
  txt = str_replace_all(txt, "\\bnursejami\\b", " nurse ")
  txt = str_replace_all(txt, "\\bnursen\\b", " nurse ")
  txt = str_replace_all(txt, "\\bnurseri\\b", " nurse ")
  
  txt = str_replace_all(txt, "\\boccurr\\b", " occur ")
  txt = str_replace_all(txt, "\\borderat\\b", " order at ")
  txt = str_replace_all(txt, "\\borderbut\\b", " order but ")
  txt = str_replace_all(txt, "\\borderedphysician\\b", " order physician ")
  txt = str_replace_all(txt, "\\borderphysician\\b", " order physician ")
  txt = str_replace_all(txt, "\\borderset\\b", " order set ")
  txt = str_replace_all(txt, "\\borderthi\\b", " order this ")
  txt = str_replace_all(txt, "\\boutpati\\b", " out patient ")
  txt = str_replace_all(txt, "\\boxycodon\\b", " oxycodone ")
  txt = str_replace_all(txt, "\\boxycontin\\b", " oxycodone ")
  txt = str_replace_all(txt, "\\boxytocin\\b", " oxycodone ")
  
  txt = str_replace_all(txt, "\\bpaient\\b", " patient ")
  txt = str_replace_all(txt, "\\bpager\\b", " page ")
  txt = str_replace_all(txt, "\\bpaitent\\b", " patient ")
  txt = str_replace_all(txt, "\\bpateint\\b", " patient ")
  txt = str_replace_all(txt, "\\bpatientdc\\b", " patient dc ")
  txt = str_replace_all(txt, "\\bpatientheparin\\b", " patient theparin ")
  txt = str_replace_all(txt, "\\bpatientn\\b", " patient ")
  txt = str_replace_all(txt, "\\bphamacist\\b", " pharmacist ")
  txt = str_replace_all(txt, "\\bpharmaci\\b", " pharmacy ")
  txt = str_replace_all(txt, "\\bpharmacist\\b", " pharmacist ")
  txt = str_replace_all(txt, "\\bpharmacistpharmaci\\b", " pharmacy pharmacist ")
  txt = str_replace_all(txt, "\\bpharmacypolici\\b", " pharmacy policy ")
  #txt = str_replace_all(txt, "\\bpt\\b", " patient ")
  txt = str_replace_all(txt, "\\bresid\\b", " resident ")
  
  
  txt = str_replace_all(txt, "\\bphysican\\b", " physician ")
  txt = str_replace_all(txt, "\\bphysicianchangir\\b", " physician ")
  txt = str_replace_all(txt, "\\bphysicianeben\\b", " physician ")
  txt = str_replace_all(txt, "\\bphysicianfeld\\b", " physician ")
  txt = str_replace_all(txt, "\\bphysicianlab\\b", " physician lab ")
  txt = str_replace_all(txt, "\\bphysicianlazarousmicuphysician\\b", " physician ")
  txt = str_replace_all(txt, "\\bphysicianmay\\b", " patient ")
  
  txt = str_replace_all(txt, "\\balllerg\\b", " alllergy ")
  txt = str_replace_all(txt, "\\bamri\\b", " mari ")
  txt = str_replace_all(txt, "\\bassoic\\b", " associ ")
  txt = str_replace_all(txt, "\\bassocait\\b", " associ ")
  txt = str_replace_all(txt, "\\bassoc\\b", " associ ")
  txt = str_replace_all(txt, "\\bbackalso\\b", " back also ")
  txt = str_replace_all(txt, "\\bbandagedtdap\\b", " bandag tdap ")
  txt = str_replace_all(txt, "\\bbbe\\b", " bbp ")
  txt = str_replace_all(txt, "\\bbent\\b", " bend ")
  txt = str_replace_all(txt, "\\bbiohazrd\\b", " biohazard ")
  txt = str_replace_all(txt, "\\bbitten\\b", " bite ")
  txt = str_replace_all(txt, "\\bbit\\b", " bite ")
  txt = str_replace_all(txt, "\\bbuilt\\b", " build ")
  txt = str_replace_all(txt, "\\bchnage\\b", " change ")
  txt = str_replace_all(txt, "\\bcleans\\b", " clean ")
  txt = str_replace_all(txt, "\\bcloser\\b", " close ")
  txt = str_replace_all(txt, "\\bdermatolog\\b", " dermatologist ")
  txt = str_replace_all(txt, "\\bdusti\\b", " dust ")
  txt = str_replace_all(txt, "\\bepsosd\\b", " episod ")
  txt = str_replace_all(txt, "\\bexposur\\b", " expos ")
  txt = str_replace_all(txt, "\\bfacial\\b", " face ")
  
  txt = str_replace_all(txt, "\\bfeel\\b", " felt ")
  txt = str_replace_all(txt, "\\bfingur\\b", " finger ")
  txt = str_replace_all(txt, "\\bfeet\\b", " foot ")
  txt = str_replace_all(txt, "\\bhosptial\\b", " hospit ")
  
  txt = str_replace_all(txt, "\\binfecti\\b", " infect ")
  txt = str_replace_all(txt, "\\birit\\b", " irrit ")
  txt = str_replace_all(txt, "\\bl\\b", " left ")
  txt = str_replace_all(txt, "\\blbs\\b", " lb ")
  txt = str_replace_all(txt, "\\blge\\b", " larg ")
  txt = str_replace_all(txt, "\\bloosen\\b", " loos ")
  txt = str_replace_all(txt, "\\bnearbi\\b", " near ")
  txt = str_replace_all(txt, "\\bx ray\\b", " xray ")
  
  txt = str_replace_all(txt, "\\bntil\\b", " until ")
  txt = str_replace_all(txt, "\\bpaingo\\b", " pain go ")
  txt = str_replace_all(txt, "\\bparrafin\\b", " paraffin ")
  txt = str_replace_all(txt, "\\bperfor\\b", " perform ")
  txt = str_replace_all(txt, "\\bpinkey\\b", " pinki ")
  txt = str_replace_all(txt, "\\br\\b", " right ")
  txt = str_replace_all(txt, "\\breciev\\b", " receiv ")
  
  txt = str_replace_all(txt, "\\brt\\b", " right ")
  txt = str_replace_all(txt, "\\bshelv\\b", " shelf ")
  txt = str_replace_all(txt, "\\bsplill\\b", " spill ")
  txt = str_replace_all(txt, "\\bpt\\b", " patient ")
  txt = str_replace_all(txt, "\\bstetcher\\b", " stretcher ")
  txt = str_replace_all(txt, "\\bswung\\b", " swing ")
  txt = str_replace_all(txt, "\\bswollen\\b", " swell ")
  
  txt = str_replace_all(txt, "\\bswung\\b", " swing ")
  txt = str_replace_all(txt, "\\bthought\\b", " think ")
  txt = str_replace_all(txt, "\\bthrobi\\b", " throb ")
  txt = str_replace_all(txt, "\\btoeafter\\b", " toe after ")
  txt = str_replace_all(txt, "\\btrashbag\\b", " trash bag ")
  txt = str_replace_all(txt, "\\bxr\\b", " xray ")
  txt = str_replace_all(txt, "\\bthrought\\b", " through ")
  
  
  txt = str_replace_all(txt, "\\babgsvcresult\\b", " abgsvc result ")
  txt = str_replace_all(txt, "\\bdcmedic\\b", " dc medic ")
  txt = str_replace_all(txt, "\\bfloormedicin\\b", " floor medicin ")
  txt = str_replace_all(txt, "\\bmedicalcareabov\\b", " medicin care abov ")
  txt = str_replace_all(txt, "\\bmedicalhistori\\b", " medicin histori ")
  txt = str_replace_all(txt, "\\bhismediport\\b", " his mediport ")
  txt = str_replace_all(txt, "\\bimmediatelyord\\b", " immediate order ")
  txt = str_replace_all(txt, "\\bimmediatelysuff\\b", " immediate suffer ")
  txt = str_replace_all(txt, "\\bimmediatelyto\\b", " immediate to ")
  txt = str_replace_all(txt, "\\bimmeditali\\b", " immediate ")
  txt = str_replace_all(txt, "\\binmedi\\b", " immediate ")
  txt = str_replace_all(txt, "\\bknownimmedi\\b", " know immediate ")
  txt = str_replace_all(txt, "\\bfallhistori\\b", " fall histori ")
  txt = str_replace_all(txt, "\\bmedicalhistori\\b", " medic histori ")
  txt = str_replace_all(txt, "\\bhitthe\\b", " hit ")
  txt = str_replace_all(txt, "\\banypain\\b", " any pain ")
  txt = str_replace_all(txt, "\\bThepatient\\b", " patient ")
  txt = str_replace_all(txt, "\\bpt\\b", " patient ")
  txt = str_replace_all(txt, "\\bfactthat\\b", " fact ")
  txt = str_replace_all(txt, "\\bmedialmalleolus\\b", " medial malleolus ")
  txt = str_replace_all(txt, "\\bmedicalcareabov\\b", " medic care abov ")
  txt = str_replace_all(txt, "\\binmar\\b", " in mar ")
  txt = str_replace_all(txt, "\\bmedicalintervent\\b", " medic intervent ")
  txt = str_replace_all(txt, "\\bmedicalproblem\\b", " medic problem ")
  txt = str_replace_all(txt, "\\bmedicalpt\\b", " medic patient ")
  txt = str_replace_all(txt, "\\bmedicationa\\b", " medication ")
  txt = str_replace_all(txt, "\\bmedicationdraw\\b", " medication draw ")
  txt = str_replace_all(txt, "\\bmedicationat\\b", " medication at ")
  txt = str_replace_all(txt, "\\bmedicationimmedi\\b", " medication immediate ")
  txt = str_replace_all(txt, "\\bmedicationpati\\b", " medication patient ")
  txt = str_replace_all(txt, "\\bmedicationha\\b", " medication ha ")
  txt = str_replace_all(txt, "\\badministrationb\\b", " administration ")
  txt = str_replace_all(txt, "\\bmedicationha\\b", " medication has ")
  txt = str_replace_all(txt, "\\bmedicationr\\b", " medication ")
  txt = str_replace_all(txt, "\\bmedicationspotentiallydiscontinu\\b", " medication potential discontinu ")
  txt = str_replace_all(txt, "\\bmedicationwa\\b", " medication was ")
  txt = str_replace_all(txt, "\\bmedicaton\\b", " medication ")
  txt = str_replace_all(txt, "\\bmedicinesnow\\b", " medicin now ")
  txt = str_replace_all(txt, "\\bneedleimmedi\\b", " needle immediate ")
  txt = str_replace_all(txt, "\\bofmedicin\\b", " of medicin ")
  txt = str_replace_all(txt, "\\bpresumedimmobl\\b", " presume immobil ")
  txt = str_replace_all(txt, "\\bwheelchairimmediatelyi\\b", " wheelchair immediate ")
  txt = str_replace_all(txt, "\\wheelchairimmedi\\b", " wheelchair immediate ")
  txt = str_replace_all(txt, "\\babdomenul\\b", " abdomen ")
  txt = str_replace_all(txt, "\\babouta\\b", " about ")
  txt = str_replace_all(txt, "\\bpatientwer\\b", " patient were ")
  txt = str_replace_all(txt, "\\babouthow\\b", " about how ")
  txt = str_replace_all(txt, "\\baboutissu\\b", " about tissu ")
  txt = str_replace_all(txt, "\\babrasian\\b", " abras ")
  txt = str_replace_all(txt, "\\babrass\\b", " abras ")
  txt = str_replace_all(txt, "\\babsces\\b", " abscess ")
  txt = str_replace_all(txt, "\\babsorpt\\b", " absorb ")
  
  txt = str_replace_all(txt, "\\baccepterd\\b", " accept ")
  txt = str_replace_all(txt, "\\bacces\\b", " access ")
  txt = str_replace_all(txt, "\\baccidentallymislabel\\b", " accident mislabel ")
  txt = str_replace_all(txt, "\\baccn\\b", " account ")
  txt = str_replace_all(txt, "\\acct\\b", " account ")
  txt = str_replace_all(txt, "\\baccuamx\\b", " accumax ")
  txt = str_replace_all(txt, "\\baccuchek\\b", " accucheck ")
  txt = str_replace_all(txt, "\\baccusedm\\b", " accuse ")
  txt = str_replace_all(txt, "\\baccur\\b", " accuraci ")
  txt = str_replace_all(txt, "\\baccuracyshould\\b", " accuraci should ")
  txt = str_replace_all(txt, "\\baccusedm\\b", " accuse ")
  txt = str_replace_all(txt, "\\bacetylcystein\\b", " acetycystein ")
  txt = str_replace_all(txt, "\\bachair\\b", " chair ")
  txt = str_replace_all(txt, "\\bachieveh\\b", " achiev ")
  txt = str_replace_all(txt, "\\bachill\\b", " achilli ")
  txt = str_replace_all(txt, "\\bactuali\\b", " actual ")
  txt = str_replace_all(txt, "\\bactualintub\\b", " actual tub ")
  txt = str_replace_all(txt, "\\bactuallydischarg\\b", " actual discharg ")
  txt = str_replace_all(txt, "\\adamawouldcheck\\b", " adam would check ")
  txt = str_replace_all(txt, "\\badditonali\\b", " addit ")
  
  txt = str_replace_all(txt, "\\baddmit\\b", " admit ")
  txt = str_replace_all(txt, "\\baddres\\b", " address ")
  txt = str_replace_all(txt, "\\baddressograph\\b", " address graph ")
  txt = str_replace_all(txt, "\\badm\\b", " admission ")
  txt = str_replace_all(txt, "\\badminister\\b", " admininist ")
  txt = str_replace_all(txt, "\\badministratoriri\\b", " administrator ")
  txt = str_replace_all(txt, "\\badministratornotifi\\b", " administrator notifi ")
  txt = str_replace_all(txt, "\\badministratoroff\\b", " administrator off ")
  txt = str_replace_all(txt, "\\badmissin\\b", " admission ")
  txt = str_replace_all(txt, "\\badmissionwhil\\b", " admission while ")
  txt = str_replace_all(txt, "\\badmite\\b", " admit ")
  txt = str_replace_all(txt, "\\badmitr\\b", " admit ")
  txt = str_replace_all(txt, "\\badmittingb\\b", " admit ")
  txt = str_replace_all(txt, "\\badmmit\\b", " admit ")
  txt = str_replace_all(txt, "\\badmnist\\b", " administer ")
  txt = str_replace_all(txt, "\\bapati\\b", " a patient ")
  txt = str_replace_all(txt, "\\bpatientinterview\\b", " patient interview ")
  txt = str_replace_all(txt, "\\badrenelectomi\\b", " adrenalectomi ")
  txt = str_replace_all(txt, "\\baduit\\b", " audit ")
  txt = str_replace_all(txt, "\\bafrom\\b", " a from ")
  
  txt = str_replace_all(txt, "\\baftera\\b", " after a ")
  txt = str_replace_all(txt, "\\bafterabout\\b", " after about ")
  txt = str_replace_all(txt, "\\baftercalland\\b", " after call and ")
  txt = str_replace_all(txt, "\\baftermak\\b", " after mak ")
  txt = str_replace_all(txt, "\\bageindetermin\\b", " age indetermin ")
  txt = str_replace_all(txt, "\\bagitt\\b", " agitate ")
  txt = str_replace_all(txt, "\\bacccess\\b", " access ")
  txt = str_replace_all(txt, "\\baccompni\\b", " accompani ")
  txt = str_replace_all(txt, "\\bacc\\b", " account ")
  txt = str_replace_all(txt, "\\bacct\\b", " account ")
  txt = str_replace_all(txt, "\\bactiveplan\\b", " active plan ")
  txt = str_replace_all(txt, "\\badamawouldcheck\\b", " adama would check ")
  txt = str_replace_all(txt, "\\badmininist\\b", " administer ")
  txt = str_replace_all(txt, "\\badministrationi\\b", " administration ")
  txt = str_replace_all(txt, "\\badmissionsacrum\\b", " admission ")
  txt = str_replace_all(txt, "\\badmissionthen\\b", " admission ")
  txt = str_replace_all(txt, "\\badvancementsubsequ\\b", " advanc ")
  txt = str_replace_all(txt, "\\badvicea\\b", " advise ")
  txt = str_replace_all(txt, "\\badvis\\b", " advise ")
  txt = str_replace_all(txt, "\\badvocay\\b", " advocate ")
  txt = str_replace_all(txt, "\\baffectoutcom\\b", " affect outcom ")
  txt = str_replace_all(txt, "\\bagofrom\\b", " ago from ")
  txt = str_replace_all(txt, "\\balsopati\\b", " also patient ")
  txt = str_replace_all(txt, "\\banaphylaxsi\\b", " anaphylact ")
  txt = str_replace_all(txt, "\\bandno\\b", " and no ")
  
  
  txt = str_replace_all(txt, "\\bandsh\\b", " and she ")
  txt = str_replace_all(txt, "\\bandspok\\b", " and spoke ")
  txt = str_replace_all(txt, "\\bandwa\\b", " and was ")
  txt = str_replace_all(txt, "\\bantiplatelet\\b", " anticoagul ")
  txt = str_replace_all(txt, "\\bapast\\b", " a past ")
  txt = str_replace_all(txt, "\\bappt\\b", " appoint ")
  txt = str_replace_all(txt, "\\barmband\\b", " arm band ")
  txt = str_replace_all(txt, "\\basses\\b", " assess ")
  txt = str_replace_all(txt, "\\bassisst\\b", " assist ")
  txt = str_replace_all(txt, "\\bassistanc\\b", " assist ")
  txt = str_replace_all(txt, "\\batapproxim\\b", " at appropriate ")
  txt = str_replace_all(txt, "\\battemp\\b", " attempt ")
  txt = str_replace_all(txt, "\\bawok\\b", " awak ")
  txt = str_replace_all(txt, "\\bbecuas\\b", " becaus ")
  txt = str_replace_all(txt, "\\bbedand\\b", " bed and ")
  
  txt = str_replace_all(txt, "\\bbegun\\b", " begin ")
  
  txt = str_replace_all(txt, "\\bbegan\\b", " begin ")
  txt = str_replace_all(txt, "\\bbirthtim\\b", " birth time ")
  txt = str_replace_all(txt, "\\bbloodi\\b", " blood ")
  txt = str_replace_all(txt, "\\bbmpb\\b", " bmp ")
  txt = str_replace_all(txt, "\\bbroken\\b", " broke ")
  txt = str_replace_all(txt, "\\bbruse\\b", " bruis ")
  txt = str_replace_all(txt, "\\bbutt\\b", " buttock ")
  txt = str_replace_all(txt, "\\bcalledand\\b", " call ")
  txt = str_replace_all(txt, "\\bcannot\\b", " can not ")
  txt = str_replace_all(txt, "\\bcant\\b", " can not ")
  txt = str_replace_all(txt, "\\bcardiologist\\b", " cardiolog ")
  txt = str_replace_all(txt, "\\bcaregiv\\b", " care give ")
  txt = str_replace_all(txt, "\\bcaretak\\b", " care take ")
  txt = str_replace_all(txt, "\\baccucheck\\b", " accu check ")
  txt = str_replace_all(txt, "\\bchek\\b", " check ")
  txt = str_replace_all(txt, "\\bclarif\\b", " clarifi ")
  txt = str_replace_all(txt, "\\bcomparison\\b", " compar ")
  txt = str_replace_all(txt, "\\bcompliant\\b", " complianc ")
  txt = str_replace_all(txt, "\\bconsid\\b", " consider ")
  txt = str_replace_all(txt, "\\bcontaminatedr\\b", " contaminate ")
  txt = str_replace_all(txt, "\\bcontamin\\b", " contaminate ")
  txt = str_replace_all(txt, "\\bcooler\\b", " cool ")
  txt = str_replace_all(txt, "\\bcouldn\\b", " could not ")
  txt = str_replace_all(txt, "\\bcouldnt\\b", " could not ")
  txt = str_replace_all(txt, "\\bdecid\\b", " decision ")
  txt = str_replace_all(txt, "\\bdecis\\b", " decision ")
  txt = str_replace_all(txt, "\\bandfel\\b", " and fall ")
  txt = str_replace_all(txt, "\\bdiagnos\\b", " diagnose ")
  txt = str_replace_all(txt, "\\bdialys\\b", " dialysis ")
  txt = str_replace_all(txt, "\\bdialysi\\b", " dialysis ")
  txt = str_replace_all(txt, "\\bdialyz\\b", " dialysis ")
  txt = str_replace_all(txt, "\\bknoee\\b", " knee ")
  txt = str_replace_all(txt, "\\bdosag\\b", " dose ")
  txt = str_replace_all(txt, "\\bbilat\\b", " bilater ")
  txt = str_replace_all(txt, "\\bcomplainingof\\b", " complain of ")
  txt = str_replace_all(txt, "\\bconsciou\\b", " conscious ")
  txt = str_replace_all(txt, "\\bconsumpt\\b", " consum ")
  txt = str_replace_all(txt, "\\bcrossmatch\\b", " cross match ")
  txt = str_replace_all(txt, "\\bcxr\\b", " chest xray ")
  
  # txt = str_replace_all(txt, "\\pcxr\\b", " portable chest xray ")
  # txt = str_replace_all(txt, "\\bdeath\\b", " dead ")
  # txt = str_replace_all(txt, "\\bdefinit\\b", " defin ")
  
  txt = str_replace_all(txt, "\\bdepartur\\b", " depart ")
  txt = str_replace_all(txt, "\\bdescrib\\b", " descript ")
  txt = str_replace_all(txt, "\\bdiagnos\\b", " diagnose ")
  txt = str_replace_all(txt, "\\bdisori\\b", " disorient ")
  txt = str_replace_all(txt, "\\bdr\\b", " doctor ")
  txt = str_replace_all(txt, "\\bdrank\\b", " drink ")
  txt = str_replace_all(txt, "\\bdriven\\b", " drive ")
  txt = str_replace_all(txt, "\\bdrove\\b", " drive ")
  txt = str_replace_all(txt, "\\bdtiright\\b", " dti right ")
  txt = str_replace_all(txt, "\\bechocardiogram\\b", " echo cardiogram ")
  txt = str_replace_all(txt, "\\bemergencydepart\\b", " emergency depart ")
  txt = str_replace_all(txt, "\\bendo\\b", " endoscopi ")
  txt = str_replace_all(txt, "\\benviroment\\b", " environment ")
  txt = str_replace_all(txt, "\\benviron\\b", " environment ")
  txt = str_replace_all(txt, "\\beveyr\\b", " everi ")
  txt = str_replace_all(txt, "\\bextub\\b", " extubate ")
  txt = str_replace_all(txt, "\\bextubatedb\\b", " extubate ")
  txt = str_replace_all(txt, "\\bfaster\\b", " fast ")
  txt = str_replace_all(txt, "\\bfaulti\\b", " fault ")
  txt = str_replace_all(txt, "\\bfellbackward\\b", " fell backward ")
  txt = str_replace_all(txt, "\\bfemor\\b", " femur ")
  txt = str_replace_all(txt, "\\btoth\\b", " to the ")
  txt = str_replace_all(txt, "\\bfoor\\b", " floor ")
  txt = str_replace_all(txt, "\\baddressograph\\b", " address graph ")
  txt = str_replace_all(txt, "\\bformulari\\b", " formula ")
  txt = str_replace_all(txt, "\\bfromth\\b", " from ")
  txt = str_replace_all(txt, "\\bfulli\\b", " full ")
  
  txt = str_replace_all(txt, "\\bgase\\b", " gas ")
  txt = str_replace_all(txt, "\\bgass\\b", " gas ")
  txt = str_replace_all(txt, "\\bgiven\\b", " give ")
  txt = str_replace_all(txt, "\\bgonead\\b", " gone ")
  txt = str_replace_all(txt, "\\bheard\\b", " hear ")
  txt = str_replace_all(txt, "\\bhistor\\b", " histori ")
  txt = str_replace_all(txt, "\\bhistoryin\\b", " histori ")
  txt = str_replace_all(txt, "\\bhypoglycem\\b", " hypoglycemia ")
  txt = str_replace_all(txt, "\\bhypertens\\b", " hypotens ")
  txt = str_replace_all(txt, "\\bicc\\b", " icu ")
  txt = str_replace_all(txt, "\\bident\\b", " identify ")
  txt = str_replace_all(txt, "\\bincid\\b", " incident ")
  txt = str_replace_all(txt, "\\bincuid\\b", " includ ")
  txt = str_replace_all(txt, "\\binde\\b", " indeed ")
  txt = str_replace_all(txt, "\\binfo\\b", " inform ")
  txt = str_replace_all(txt, "\\binjuir\\b", " injury ")
  txt = str_replace_all(txt, "\\binterven\\b", " intervent ")
  txt = str_replace_all(txt, "\\bischemia\\b", " ischem ")
  txt = str_replace_all(txt, "\\bitold\\b", " i told ")
  
  txt = str_replace_all(txt, "\\bjacki\\b", "")
  txt = str_replace_all(txt, "\\bjackson\\b", "")
  txt = str_replace_all(txt, "\\bjacob\\b", "")
  txt = str_replace_all(txt, "\\bjame\\b", "")
  txt = str_replace_all(txt, "\\bjami\\b", "")
  txt = str_replace_all(txt, "\\bjan\\b", "")
  txt = str_replace_all(txt, "\\bjanet\\b", "")
  txt = str_replace_all(txt, "\\bjean\\b", "")
  txt = str_replace_all(txt, "\\bjeann\\b", "")
  txt = str_replace_all(txt, "\\bjeanni\\b", "")
  txt = str_replace_all(txt, "\\bjeff\\b", "")
  txt = str_replace_all(txt, "\\bjen\\b", "")
  txt = str_replace_all(txt, "\\bjennif\\b", "")
  txt = str_replace_all(txt, "\\bjenna\\b", "")
  txt = str_replace_all(txt, "\\bjessica\\b", "")
  txt = str_replace_all(txt, "\\bjim\\b", "")
  txt = str_replace_all(txt, "\\bjoan\\b", "")
  txt = str_replace_all(txt, "\\bjohn\\b", "")
  txt = str_replace_all(txt, "\\bjohnson\\b", "")
  txt = str_replace_all(txt, "\\bjonathan\\b", "")
  txt = str_replace_all(txt, "\\bjone\\b", "")
  txt = str_replace_all(txt, "\\bjordan\\b", "")
  txt = str_replace_all(txt, "\\bjoseph\\b", "")
  txt = str_replace_all(txt, "\\bjosh\\b", "")
  txt = str_replace_all(txt, "\\bjoyc\\b", "")
  txt = str_replace_all(txt, "\\bjudi\\b", "")
  txt = str_replace_all(txt, "\\bjuli\\b", "")
  txt = str_replace_all(txt, "\\bkaren\\b", "")
  txt = str_replace_all(txt, "\\bkate\\b", "")
  txt = str_replace_all(txt, "\\bkatherin\\b", "")
  txt = str_replace_all(txt, "\\bkathi\\b", "")
  txt = str_replace_all(txt, "\\bkathleen\\b", "")
  txt = str_replace_all(txt, "\\bkathryn\\b", "")
  txt = str_replace_all(txt, "\\bkati\\b", "")
  txt = str_replace_all(txt, "\\bkaushal\\b", "")
  txt = str_replace_all(txt, "\\bkelli\\b", "")
  txt = str_replace_all(txt, "\\bkelsey\\b", "")
  txt = str_replace_all(txt, "\\bkenneth\\b", "")
  txt = str_replace_all(txt, "\\bkimber\\b", "")
  txt = str_replace_all(txt, "\\bkim\\b", "")
  txt = str_replace_all(txt, "\\bkristen\\b", "")
  txt = str_replace_all(txt, "\\blaura\\b", "")
  txt = str_replace_all(txt, "\\blauren\\b", "")
  txt = str_replace_all(txt, "\\blauri\\b", "")
  txt = str_replace_all(txt, "\\blesli\\b", "")
  txt = str_replace_all(txt, "\\blinda\\b", "")
  txt = str_replace_all(txt, "\\blindsey\\b", "")
  txt = str_replace_all(txt, "\\blori\\b", "")
  txt = str_replace_all(txt, "\\bmargaret\\b", "")
  txt = str_replace_all(txt, "\\bmaria\\b", "")
  txt = str_replace_all(txt, "\\bmari\\b", "")
  txt = str_replace_all(txt, "\\bmaria\\b", "")
  txt = str_replace_all(txt, "\\bmartin\\b", "")
  txt = str_replace_all(txt, "\\bmatt\\b", "")
  txt = str_replace_all(txt, "\\bmccoy\\b", "")
  txt = str_replace_all(txt, "\\bmcmillan\\b", "")
  txt = str_replace_all(txt, "\\bmedstar\\b", "")
  txt = str_replace_all(txt, "\\bmegan\\b", "")
  txt = str_replace_all(txt, "\\bmelani\\b", "")
  txt = str_replace_all(txt, "\\bmelissa\\b", "")
  txt = str_replace_all(txt, "\\bmichael\\b", "")
  txt = str_replace_all(txt, "\\bmichell\\b", "")
  txt = str_replace_all(txt, "\\bmiriam\\b", "")
  txt = str_replace_all(txt, "\\bmitchel\\b", "")
  txt = str_replace_all(txt, "\\bmonica\\b", "")
  txt = str_replace_all(txt, "\\bmoniqu\\b", "")
  txt = str_replace_all(txt, "\\bnatali\\b", "")
  txt = str_replace_all(txt, "\\bmorgan\\b", "")
  txt = str_replace_all(txt, "\\bnguyen\\b", "")
  txt = str_replace_all(txt, "\\bobrien\\b", "")
  txt = str_replace_all(txt, "\\bpatrick\\b", "")
  
  txt = str_replace_all(txt, "\\bpatric\\b", "")
  txt = str_replace_all(txt, "\\bpaul\\b", "")
  txt = str_replace_all(txt, "\\bpeter\\b", "")
  txt = str_replace_all(txt, "\\bpeterson\\b", "")
  txt = str_replace_all(txt, "\\brachel\\b", "")
  txt = str_replace_all(txt, "\\bricardo\\b", "")
  txt = str_replace_all(txt, "\\brichard\\b", "")
  
  txt = str_replace_all(txt, "\\blightheaded\\b", " lighthead ")
  txt = str_replace_all(txt, "\\blose\\b", " loss ")
  txt = str_replace_all(txt, "\\blost\\b", " loss ")
  txt = str_replace_all(txt, "\\bmainten\\b", " maintain ")
  txt = str_replace_all(txt, "\\bmanger\\b", " manag ")
  txt = str_replace_all(txt, "\\bmarron\\b", " maroon ")
  txt = str_replace_all(txt, "\\bmeant\\b", " mean ")
  txt = str_replace_all(txt, "\\bmedicalhistori\\b", " medic histori ")
  txt = str_replace_all(txt, "\\bmistaken\\b", " mistak ")
  txt = str_replace_all(txt, "\\bmonit\\b", " monitor ")
  txt = str_replace_all(txt, "\\bmris\\b", " mri ")
  txt = str_replace_all(txt, "\\bmulti\\b", " multipl ")
  txt = str_replace_all(txt, "\\bmulitpl\\b", " multipl ")
  txt = str_replace_all(txt, "\\bmusclespasm\\b", " muscl spasm ")
  txt = str_replace_all(txt, "\\bnauseat\\b", " nausea ")
  txt = str_replace_all(txt, "\\bneonatologist\\b", " neonat ")
  txt = str_replace_all(txt, "\\bnephrolog\\b", " nephrologist ")
  txt = str_replace_all(txt, "\\bnoiseand\\b", " nois ")
  txt = str_replace_all(txt, "\\bnotbeen\\b", " not been ")
  txt = str_replace_all(txt, "\\bnotat\\b", " not at ")
  txt = str_replace_all(txt, "\\bnotifiy\\b", " notify ")
  txt = str_replace_all(txt, "\\bnurs\\b", " nurse ")
  txt = str_replace_all(txt, "\\bnursea\\b", " nurse ")
  txt = str_replace_all(txt, "\\boccas\\b", " occasion ")
  txt = str_replace_all(txt, "\\boccass\\b", " occasion ")
  txt = str_replace_all(txt, "\\boncologist\\b", " oncolog ")
  txt = str_replace_all(txt, "\\bpatientand\\b", " patient and ")
  txt = str_replace_all(txt, "\\bpatientdid\\b", " patient did ")
  txt = str_replace_all(txt, "\\bpatientha\\b", " patient ")
  txt = str_replace_all(txt, "\\bpatienthad\\b", " patient had ")
  txt = str_replace_all(txt, "\\bpatientreport\\b", " patient report ")
  txt = str_replace_all(txt, "\\bpatientwa\\b", " patient was ")
  txt = str_replace_all(txt, "\\bpatiet\\b", " patient ")
  txt = str_replace_all(txt, "\\bpatinet\\b", " patient ")
  txt = str_replace_all(txt, "\\bpediatrician\\b", " pediatr ")
  txt = str_replace_all(txt, "\\bpelvi\\b", " pelvic ")
  txt = str_replace_all(txt, "\\bpharmacist\\b", " pharmacy ")
  txt = str_replace_all(txt, "\\bphlebotomi\\b", " phlebotomist ")
  txt = str_replace_all(txt, "\\bpregnanc\\b", " pregnant ")
  txt = str_replace_all(txt, "\\bradiologist\\b", " radiolog ")
  txt = str_replace_all(txt, "\\breceivedtwo\\b", " receiv ")
  txt = str_replace_all(txt, "\\brecevi\\b", " receiv ")
  txt = str_replace_all(txt, "\\bredrawn\\b", " redraw ")
  txt = str_replace_all(txt, "\\breeduc\\b", " reduc ")
  txt = str_replace_all(txt, "\\bregistr\\b", " regist ")
  txt = str_replace_all(txt, "\\bremaind\\b", " remain ")
  txt = str_replace_all(txt, "\\bresidualright\\b", " residu right ")
  txt = str_replace_all(txt, "\\brestraint\\b", " restrain ")
  #txt = str_replace_all(txt, "\\b\\b[0-9]{1,4} ?am\\b| \\b[0-9]{1,2}\\:[0-9]{2} ?am\\b", " time am ")
  return(txt)
}




replaceGsub <- function(text) {
  # run before using tolower
  text = gsub("DR.", "physician", text, fixed = TRUE)
  text = gsub("Dr.", "physician", text, fixed = TRUE)
  text = gsub("DrD.", "DrD", text, fixed = TRUE)
  text = gsub("dr.", "physician", text, fixed = TRUE)
  text = gsub("PT.", "pt", text, fixed = TRUE)
  text = gsub("Pt.", "Pt", text, fixed = TRUE)
  text = gsub("pt.", "pt", text, fixed = TRUE)
  text = gsub("C.D", "CD", text, fixed = TRUE)
  text = gsub("C.d", "Cd", text, fixed = TRUE)
  text = gsub("c.d", "cd", text, fixed = TRUE)
  text = gsub("y.o.", "yo", text, fixed = TRUE)
  text = gsub("o.r.", "or", text, fixed = TRUE)
  text = gsub("O.R.", "OR", text, fixed = TRUE)
  text = gsub("E.R.pt", "er pt", text , fixed = TRUE) # , fixed = TRUE
  text = gsub("E.R.pt.", "er pt", text , fixed = TRUE) #, fixed = TRUE
  text = gsub("E.R.pt.", "er pt", text , fixed = TRUE) # , fixed = TRUE
  text = gsub(".E.R.pt.", "er pt", text , fixed = TRUE) # 
  text = gsub("e.R. pt.", "er pt", text , fixed = TRUE) #
  text = gsub("e.r pt.", "er pt", text , fixed = TRUE) # 
  text = gsub("e.r.pt.", "er pt", text , fixed = TRUE) #
  text = gsub("E.R", "ER", text , fixed = TRUE) # 
  text = gsub("e.r.", "er", text, fixed = TRUE)
  text = gsub("e.r", "er", text, fixed = TRUE)
  text = gsub("E.R.", "ER", text , fixed = TRUE) # , fixed = TRUE
  text = gsub("M.r.", "Mr", text, fixed = TRUE)
  text = gsub("I.R.", "ir", text, fixed = TRUE)
  text = gsub("Mr.", "Mr", text, fixed = TRUE)
  #text = str_replace_all(text, "Mrs.", "Mrs")
  text = gsub("M.s.", "Ms", text, fixed = TRUE)
  text = gsub("Ms.", "Ms", text, fixed = TRUE)
  text = gsub("P.A.", "PA", text, fixed = TRUE)
  text = gsub("p.a.", "PA", text, fixed = TRUE)
  text = gsub("\\bc/o\\b", "co", text)
  text = gsub(" d/c", "dc", text, fixed = TRUE)
  text = gsub(" d.c", "dc", text, fixed = TRUE)
  text = gsub(" R.P.", "rp", text, fixed = TRUE)
  text = gsub(" N.O", "n.o", text, fixed = TRUE)
  text = gsub(" D.E.", "de", text, fixed = TRUE)
  text = gsub(" N.P. ", "np", text, fixed = TRUE)
  text = gsub("\\babd\\b", "abdomen", text, fixed = TRUE)
  text = gsub("abd", "abdomen", text, fixed = TRUE)
  text = gsub("abdominal", "abdomen", text, fixed = TRUE)
  
  text = gsub(" aaddendum ", " addendum ", text, fixed = TRUE)
  text = gsub(" aalrm ", " alarm ", text, fixed = TRUE)
  text = gsub("aambulated", " ambulated ", text, fixed = TRUE)
  text = gsub("abdomenpt", "abdomen", text, fixed = TRUE)
  text = gsub("deliveredhowever", "delivered however", text, fixed = TRUE)
  text = gsub("abdomenomenominal", "abdominal", text, fixed = TRUE)
  text = gsub("accesswithout", "access without", text, fixed = TRUE)
  text = gsub("addressedwithin", "addressed within", text, fixed = TRUE)
  text = gsub("admnurses", "adm nurses", text, fixed = TRUE)
  text = gsub("andhospitalist", "and hospitalist", text, fixed = TRUE)
  text = gsub("andloading", "and loading", text, fixed = TRUE)
  text = gsub("andthe", "and the", text, fixed = TRUE)
  text = gsub("andthen", "and then", text, fixed = TRUE)
  text = gsub("anklexray", "ankle xray", text, fixed = TRUE)
  text = gsub("apparentlymislabeled", "apparently mislabeled", text, fixed = TRUE)
  text = gsub("applytao", "apply to", text, fixed = TRUE)
  text = gsub("approachwith", "approach with", text, fixed = TRUE)
  text = gsub("arrhythmiasagain", "arrhythmias again", text, fixed = TRUE)
  text = gsub("asequipment", "as equipment", text, fixed = TRUE)
  text = gsub("atdischarge", "at discharge", text, fixed = TRUE)
  text = gsub("awareincomplete", "aware incomplete", text, fixed = TRUE)
  text = gsub("babygirl", "baby girl", text, fixed = TRUE)
  text = gsub("barnesfailed", "barnes failed", text, fixed = TRUE)
  text = gsub("beingcompleted", "being completed", text, fixed = TRUE)
  #text = gsub("a.m.", "morning", text)
  #text = gsub("p.m.", "evening", text)
  #text = gsub("A.M.", "morning", text)
  #text = gsub("P.M.", "evening", text)
  #text = gsub("[[:digit:]]\\.[[:digit:]]", "", text)
  return (as.matrix(text))
}


clean_text <- function(txt){
  #txt = txt_precleaned
  txt = replaceGsub(txt)
  txt = tolower(as.character(txt))
  txt = str_replace_all(txt, "[(]|[)]", " ")
  txt = str_replace_all(txt, " \\b[0-9]{1,4} ?am\\b| \\b[0-9]{1,2}\\:[0-9]{2} ?am\\b", " time am ")
  txt = str_replace_all(txt, " [0-9]{1,4} ?pm|[0-9]{1,2}\\:[0-9]{2} ?pm", " time pm ")
  txt = str_replace_all(txt, " \\b[0-9]{1,2}\\:[0-9]{2} ?(hr|hrs|h)\\b|minut.{1,2}| \\b[0-9]{1,4} ?(h|hr)\\b", " time hr ")
  txt = str_replace_all(txt, "(at) [0-9]{4}|[0-9]{1,2} ?min.| minut.{1,2}| [0-9]{1,2} ?minute.| [0-9]{1,2}[:][0-9]{1,2}", " time ")
  txt = str_replace_all(txt, " [0-9]{1,2}[,|/|-][0-9]{1,2}[,|/|-][0-9]{2,4}| [A-Za-z]{3,4}[\\,|\\/][0-9]{1,2}[\\,|\\/][0-9]{2,4}", " date ")
  txt = str_replace_all(txt, " [0-9]{1,4}\\s?(mg|mcgs|ml)|[0-9]{2,4}\\s?(ml|mcg/ml)|[0-9]{2,4}\\s?(mcg/hr|ml/hr)|[0-9]{1,4}\\s?(tabs|tablet|tabl|tabls|tab)", " dose ")
  txt = str_replace_all(txt, " [[:digit:]]{1,2}\\.[[:digit:]]{1,2}", " num ")
  txt = str_replace_all(txt, " [[:digit:]]{1,3}[/|-][[:digit:]]{1,3}", " num ")
  txt = str_replace_all(txt, " [[:digit:]]{1,10}", " num ")
  txt = str_replace_all(txt, "[[:digit:]]", " ")
  txt = str_replace_all(txt, "(\\'s)", "")
  txt = str_replace_all(txt, "f u", " fu ")
  txt = str_replace_all(txt, '["]|[:]', " ")
  txt = str_replace_all(txt, "[\\/|\\~|]", " ")
  txt = str_replace_all(txt, '[^a-zA-Z0-9\\.\\?\\!]', " ")
  txt = str_replace_all(txt, '[^a-zA-Z0-9\\.\\?\\!]', " ")
  txt = str_replace_all(txt, '[.]', " . ")
  txt = stripWhitespace(txt)
  #txt = as.matrix(gsub('[:punct:]', "", tolower(as.character(txt))))
  return(txt)
}


makeSentArray <- function(text) {
  #text = VectorSource(text)
  #corpus = Corpus(text)
  #sList = tm_map(corpus, sent_detect)
  sList = strsplit(text, split="[.?!]")
  
  sentArray = sList[[1]]
  sentKey = matrix(0, nrow=length(sList[[1]]), ncol=3)
  sentKey[,1] = 1
  sentKey[,2] = 1:length(sList[[1]])
  sentKey[,3] = 0 + (1:length(sList[[1]]))
  
  if (length(sList) > 1) {
    for (i in 2:length(sList)) {
      sentArray = c(sentArray, sList[[i]])  
      
      lastCnt = max(sentKey[,3])
      tempKey = matrix(0, nrow=length(sList[[i]]), ncol=3)
      tempKey[,1] = i
      tempKey[,2] = 1:length(sList[[i]])
      tempKey[,3] = lastCnt + (1:length(sList[[i]]))
      
      sentKey = rbind(sentKey, tempKey)
      
    }
  }
  colnames(sentKey) = c("reportID", "sentID", "globalID")
  
  sentArray = data.frame("sents" = sentArray, "len" = nchar(sentArray))
  sentArrayAndKey = cbind(sentKey, sentArray)
  #sentKey = sentKey[which(sentArray$len>10),]
  #sentArray = sentArray[which(sentArray$len>10),]
  
  return (sentArrayAndKey)
  
}


